//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

/*

-------------------------------------------------------------------------------
   Title      : Push Button Scan Data In
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : PB_SCAN_DATA_IN.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/19
   Last Update: 2001/06/19
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : 
-------------------------------------------------------------------------------
   Used by    : Any user logic in the MicroBlaze demo board FPGA that requires
		    push button data
-------------------------------------------------------------------------------
   Description: This module decodes the push button status from data transmitted by
		    the MicroBlaze board control CPLD.

			The CPLD shifts out the data on the falling edge of PB_CLOCK_P and 
			this module latches the data on the rising edge of PB_CLOCK_P.
			
			The push buttons operate in a toggle mode, in that they are
			push-on, push-off and the status data is transmitted to the 
			FPGA only when the "ENTER" push button (PB11) has been pressed.

			If the LED associated with the push button is "ON" then the push button
			toggle status is "ON" and the status output bit is set.


	Conventions:
		All port signals are UPPER CASE.
		Active HIGH PORT signals have a P suffex
		Active LOW PORT signals have a N suffex.
		All internal signals are LOWER CASE and are active HIGH.

-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
*/



module pb_scan_data_in
(
clock,
reset,
PB_CLOCK_P,
PB_DATA_P,
pb1_status,
pb2_status,
pb3_status,
pb4_status,
pb5_status,
pb6_status,
pb7_status,
pb8_status,
pb9_status,
pb10_status
);

input 		clock;		// system clock 27 MHz
input 		reset;		// system reset
input 		PB_CLOCK_P;		// data shift clock sourced by the CPLD
input 		PB_DATA_P;		// serial push button data from CPLD
output 		pb1_status;		// pb1_status 
output 		pb2_status;		// pb2_status 
output 		pb3_status;		// pb3_status
output 		pb4_status;		// pb4_status
output 		pb5_status;		// pb5_status
output 		pb6_status;		// pb6_status
output 		pb7_status;		// pb7_status
output 		pb8_status;		// pb8_status
output 		pb9_status;		// pb9_status
output 		pb10_status;	// pb10_status

wire		pb1_status;
wire 		pb2_status;
wire 		pb3_status;
wire		pb4_status;
wire 		pb5_status;
wire		pb6_status;
wire		pb7_status;
wire		pb8_status;
wire		pb9_status;
wire		pb10_status; 

wire 		clock;
wire 		reset;
wire 		pb_data;
wire 		pb_shift_clock;

reg [9:0] 	data_shift_reg;
reg [9:0] 	data_reg;
reg [7:0] 	data_bit_count;

reg		latch_data;

// create the internal signals from the ports

assign pb_shift_clock 	= PB_CLOCK_P;
assign pb_data 		= PB_DATA_P;


// latch the data transmitted by the CPLD 
always @ (posedge reset or posedge pb_shift_clock) begin
	if (reset) begin
		data_shift_reg[9:0] <= 10'h000;
		end
	else begin
		data_shift_reg[0] <=pb_data;
		data_shift_reg[9:1] <=data_shift_reg[8:0];
		end
	end

// create a bit counter
assign temp = (reset | latch_data);
always @ (posedge temp or posedge pb_shift_clock) begin
	if (temp) begin
		data_bit_count <= 8'h00;
		end
	else begin
		data_bit_count<= data_bit_count +1;
		end
	end

// create a signal to transfer the data to a latch
always @ (posedge reset or posedge clock) begin
	if (reset) begin
		latch_data <= 1'b0;
		end
   else if (data_bit_count == 8'h0A) begin
	   latch_data <= 1'b1;
	end
	else begin
		latch_data <= 1'b0;
		end
	end



// load the received data into a data register
always @ (posedge reset or posedge latch_data) begin
	if (reset) begin
		data_reg[9:0]	<= 10'h000;	// clear data registers on power up
		end
	else begin
		data_reg[9:0]	<= data_shift_reg[9:0];	// latch the data from the shift register
		end
	end

// decode the pb data in the data register
 assign	pb1_status	= data_reg[9];
 assign	pb2_status	= data_reg[8];
 assign	pb3_status	= data_reg[7];
 assign	pb4_status 	= data_reg[6];
 assign	pb5_status 	= data_reg[5];
 assign	pb6_status	= data_reg[4];
 assign	pb7_status	= data_reg[3];
 assign	pb8_status	= data_reg[2];
 assign	pb9_status	= data_reg[1];
 assign	pb10_status	= data_reg[0];

endmodule // pb_scan_data_in


